import random
import Enemy

class Stage:
	def __init__(self, *pars):
		pass
		
	def read_from_file(self, f):
		pass

class WaitStage(Stage):
	def __init__(self, *pars):
		Stage.__init__(self, *pars)
		
		self.wait_secs = int(pars[0])

class HintStage(Stage):
	def __init__(self, *pars):
		Stage.__init__(self, *pars)
		
		self.display_time = int(pars[0])
		self.display_text = " ".join(pars[1:])

class WaveEnemyInfo:
	def __init__(self, enemy_type, freq):
		self.enemy_type = enemy_type
		self.freq = int(freq)

class WaveStage(Stage):
	def __init__(self, *pars):
		Stage.__init__(self, *pars)

		self.enemy_count = int(pars[0])
		self.group_size = int(pars[1])
		self.time_delay = int(pars[2])

		self.enemy_types = []

	def read_from_file(self, f):
		l = ""
		while l != "wave_end":
			l = f.next().strip()
			w = l.split()
			if len(w) == 0 or w[0][0] == "#":
				continue

			if w[0] != "wave_end":
				self.enemy_types.append(WaveEnemyInfo(w[0], w[1]))

class IntermissionTextinfo(Stage):
	def __init__(self, *pars):
		Stage.__init__(self, *pars)

		self.time = int(pars[0])
		self.text_lines = []
	
	def read_from_file(self, f):
		l = ""
		while l != "textinfo_end":
			l = f.next().strip()
			if len(l) == 0 or l[0]== "#":
				continue

			if l != "textinfo_end":
				self.text_lines.append(l)

class IntermissionStage(Stage):
	def __init__(self, *pars):
		Stage.__init__(self, *pars)

		self.sub_stages = []
		self.act = -1

	def read_from_file(self, f):
		l = ""
		while l != "intermission_end":
			l = f.next().strip()
			w = l.split()
			if len(w) == 0 or w[0][0] == "#":
				continue

			if w[0] != "intermission_end":
				sclass = stage_handlers[w[0]]
				stage = sclass(*w[1:])
				self.sub_stages.append(stage)
				stage.read_from_file(f)
	
	def next(self):
		self.act += 1
		if self.act >= len(self.sub_stages):
			return None

		return self.sub_stages[self.act]

class Level(Stage):
	def __init__(self, *pars):
		Stage.__init__(self, *pars)
		
		self.sub_stages = []
		self.act = -1

	def read_from_file(self, f):
		l = f.next().strip()
		w = l.split()
		while l != "level_end" and (len(w) == 0 or w[0][0] == "#"):
			l = f.next().strip()
			w = l.split()

		if l == "level_end":
			return
			
		self.name = l

		l = ""
		while l != "level_end":
			l = f.next().strip()
			w = l.split()
			if len(w) == 0 or w[0][0] == "#":
				continue

			if w[0] != "level_end":
				sclass = stage_handlers[w[0]]
				stage = sclass(*w[1:])
				self.sub_stages.append(stage)
				stage.read_from_file(f)

	def next(self):
		self.act += 1
		if self.act >= len(self.sub_stages):
			return None

		return self.sub_stages[self.act]

class FileGameFlow(Stage):
	def __init__(self, filename):
		Stage.__init__(self)

		self.sub_stages = []
		
		f = file(filename)
		self.read_from_file(f)
		f.close()

		self.act = -1

	def read_from_file(self, f):
		for l in f:
			w = l.split()
			if len(w) == 0 or w[0][0] == "#":
				continue

			sclass = stage_handlers[w[0]]
			stage = sclass(*w[1:])
			self.sub_stages.append(stage)
			stage.read_from_file(f)

	def next(self):
		self.act += 1
		if self.act >= len(self.sub_stages):
			return None

		return self.sub_stages[self.act]

class EndlessGameFlow(Stage):
	def __init__(self):
		Stage.__init__(self)

		self.act = 0

	def read_from_file(self, f):
		pass

	def next(self):
		# Generate random level
		level = Level()
		self.act += 1

		wave = WaveStage(10 + self.act * 2, 1 + self.act / 4, random.randint(5, 10))
		for etype in Enemy.enemy_types:
			wave.enemy_types.append(WaveEnemyInfo(etype, 1))

		level.sub_stages.append(wave)
		level.sub_stages.append(WaitStage(random.randint(3, 7)))

		return level
		
stage_handlers = {
	"level" : Level,
	"wave" : WaveStage,
	"wait" : WaitStage,
	"intermission" : IntermissionStage,
	"textinfo" : IntermissionTextinfo,
	"hint" : HintStage,
}

