'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "lib"
directory.
'''

import os, sys, threading

import pygame
from pygame.locals import *

import config
from ResourceManager import *
from Sprite import *
from Scene import *

def main():
	pygame.init()
	display = pygame.display.set_mode((config.SCREEN_WIDTH, config.SCREEN_HEIGHT), 0, 32)

	pygame.display.set_caption(config.WINDOW_TITLE)

	SpriteManager(config.SPRITELIST_FILE)
	ImageManager()
	SoundManager()
	FontManager()

	scene = LoadingScene(display)
	scene.run()

	config.music = config.sound_manager["141th_street_blues.ogg"]
	config.music.play(-1)

	result = 0

	while result != 4:
		scene = MainMenuScene(display)
		result = scene.run()

		if result == 0:
			# Story mode easy
			scene = GameScene(display, FileGameFlow(config.LEVELLIST_FILE))
	
		elif result == 1:
			# Story mode - hard
			scene = GameScene(display, FileGameFlow(config.LEVELLIST_FILE))

		elif result == 2:
			# Endless mode
			scene = GameScene(display, EndlessGameFlow())
	
		elif result == 3:
			# Endless mode
			scene = HowtoScene(display)

		if result != 4:
			try:
				scene.run()
			finally:
				if isinstance(scene, GameScene) and scene.mouseSampler.isAlive():
					scene.mouseSampler.stop()
					scene.mouseSampler.join()

